@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 
-# Copyright 2005-2010 By:
+# Copyright 2005-2011 By:
 # Eduardo Gonzalez, Niklas Janlert, Christoph Reiter, Antonio Riva,
-# Aymeric Mansoux
+# Aymeric Mansoux, Nick Boultbee
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -44,7 +44,7 @@ import gzip
 
 import urllib
 import urllib2
-from HTMLParser import HTMLParser
+from HTMLParser import HTMLParser, HTMLParseError
 from cStringIO import StringIO
 from xml.dom import minidom
 
@@ -52,12 +52,10 @@ import gtk
 import gobject
 import pango
 
-from quodlibet import util, qltk, config
+from quodlibet import util, qltk, config, print_w
 from quodlibet.qltk.views import AllTreeView
 from quodlibet.plugins.songsmenu import SongsMenuPlugin
-
-#switch off, so that broken search engines wont crash the whole plugin
-debug = False
+from quodlibet.parse import Pattern
 
 USER_AGENT = "Mozilla/5.0 (X11; U; Linux i686; en-US; rv:1.9.2.13) " \
     "Gecko/20101210 Iceweasel/3.6.13 (like Firefox/3.6.13)"
@@ -130,8 +128,11 @@ class BasicHTMLParser(HTMLParser, object):
         #strip script tags/content. HTMLParser doesn't handle them well
         text = "".join([p.split("script>")[-1] for p in text.split("<script")])
 
-        self.feed(text)
-        self.close()
+        try:
+            self.feed(text)
+            self.close()
+        except HTMLParseError:
+            pass
 
     def handle_starttag(self, tag, attrs):
         if self.__buffer:
@@ -148,192 +149,134 @@ class BasicHTMLParser(HTMLParser, object):
         else:
             self.data.append(['', {}, data])
 
+
 class CoverParadiseParser(BasicHTMLParser):
     """A class for searching covers from coverparadise.to"""
 
-    def __init__(self):
-        super(CoverParadiseParser, self).__init__()
-
-        self.cover_count = 0
-        self.page_step = 0
-        self.query = ''
-        self.root_url = 'http://coverparadise.to'
-
-        self.covers = []
+    ROOT_URL = 'http://coverparadise.to'
 
     def start(self, query, limit=10):
         """Start the search and return a list of covers"""
 
-        self.cover_count = 0
-        self.page_step = 0
-        enc = get_encoding(self.root_url)
-        self.query = query.decode("utf-8").encode(enc)
-        self.covers = []
+        if isinstance(query, str):
+            query = query.decode("utf-8")
 
-        #site only takes 3+ chars
+        # site only takes 3+ chars
         if len(query) < 3:
                 return []
 
-        #parse the first page
-        self.__parse_search_list(0)
+        query = query.encode(get_encoding(self.ROOT_URL))
 
-        #if there are pagenumbers: its a list
-        if self.__parse_page_num():
-            if limit > 0:
-                self.limit = min(limit, self.cover_count)
-            else:
-                self.limit = self.cover_count
+        # parse the first page
+        self.__parse_search_list(query)
 
-            self.__extract_from_list()
+        # get the max number of offsets and the step size
+        max_offset = -1
+        step_size = 0
+        for i, (tag, attr, data) in enumerate(self.data):
+            if "SimpleSearchPage" in attr.get("href", ""):
+                offset = int(attr["href"].split(",")[1].strip("' "))
+                if offset > max_offset:
+                    step_size = step_size or (offset - max_offset)
+                    max_offset = offset
 
-            #get the rest of the list
-            for offset in xrange(self.page_step, self.limit, self.page_step):
-                self.__parse_search_list(offset)
-                self.__extract_from_list()
-        #search went directly to the album page
+        if max_offset == -1:
+            # if there is no offset, this is a single result page
+            covers = self.__extract_from_single()
         else:
-            self.__extract_from_single()
+            # otherwise parse it as a list for each page
+            covers = self.__extract_from_list()
+            for offset in range(step_size, max_offset + 1, step_size):
+                if len(covers) >= limit:
+                    break
+                self.__parse_search_list(query, offset)
+                covers.extend(self.__extract_from_list())
 
-        #delete the parsing data
         del self.data
+        return covers
 
-        return self.covers
+    def __parse_search_list(self, query, offset=0):
+        post = {"SearchString": query,
+                "Page": offset,
+                "Sektion": "2",
+                }
+
+        self.parse_url(self.ROOT_URL + '/?Module=SimpleSearch', post=post)
 
     def __extract_from_single(self):
-        """The site will directly return the album page if there is only
-        one search result"""
+        covers = []
 
-        cover = {}
+        cover = None
+        for i, (tag, attr, data) in enumerate(self.data):
+            data = data.strip()
 
-        head_pos = 0
-        for i, entry in enumerate(self.data):
-            if entry[0] == 'div' and 'class' in entry[1] and \
-                entry[1]['class'] == 'BoxHeadline':
-                head_pos = i
-                break
+            if attr.get("class", "") == "ThumbDetails":
+                cover = {"source": self.ROOT_URL}
 
-        cover['name'] = ''
-        for i in xrange(head_pos + 1, len(self.data)):
-            cover['name'] += self.data[i][2]
-            if self.data[i + 1][0].strip():
-                break
+            if cover:
+                if attr.get("href"):
+                    cover["cover"] = self.ROOT_URL + attr["href"]
 
-        cover['name'] = cover['name'].strip()
+                if attr.get("src") and "thumbnail" not in cover:
+                    cover["thumbnail"] = attr["src"]
+                    if "front" not in attr.get("alt").lower():
+                        cover = None
+                        continue
 
-        if not cover['name']:
-            return
+                if attr.get("title"):
+                    cover["name"] = attr["title"]
 
-        pic_pos = 0
-        for i in xrange(head_pos, len(self.data)):
-            if self.data[i][0] == 'img' and 'src' in self.data[i][1] and \
-                self.data[i][1]['src'].find('/thumbs/') != -1:
-                pic_pos = i
-                break
+                if tag == "br":
+                    if data.endswith("px"):
+                        cover["resolution"] = data.strip("@ ")
+                    elif data.lower().endswith("b"):
+                        cover["size"] = data.strip("@ ")
 
-        cover['thumbnail'] = self.data[pic_pos][1]['src']
+                if len(cover.keys()) >= 6:
+                    covers.append(cover)
+                    cover = None
 
-        cover['cover'] = cover['thumbnail'].replace('/thumbs/', '/')
-
-        sub_data = []
-        for i in xrange(pic_pos + 1, len(self.data)):
-            if self.data[i][0] == 'tr':
-                break
-            if not self.data[i][1] and self.data[i][2].strip():
-                sub_data.append(self.data[i])
-
-        cover['resolution'] = sub_data[-2][2].split(':')[1].strip()
-
-        cover['size'] = sub_data[-3][2].split(':')[1].strip().replace(',', '.')
-
-        cover['source'] = self.root_url
-
-        self.covers.append(cover)
-
-    def __parse_search_list(self, offset):
-        """Will parse a specific page of a search"""
-
-        search_url = self.root_url + '/?Module=SimpleSearch'
-        #Sektion 2 is for audio, Page is a search result offset
-        post_dic = {'SearchString' : self.query, 'Page': offset, 'Sektion' : 2}
-
-        self.parse_url(search_url, post=post_dic)
+        return covers
 
     def __extract_from_list(self):
-        """Extracts all the needed information from the already parsed
-        search result page and adds the found cover to the list."""
+        covers = []
 
-        table_pos = []
-        for i, entry in enumerate(self.data):
-            if entry[0] == 'div' and 'class' in entry[1] and \
-                entry[1]['class'] == 'ResultScroller':
-                table_pos.append(i)
+        cover = None
+        old_data = ""
+        last_entry = ""
+        for i, (tag, attr, data) in enumerate(self.data):
+            data = data.strip()
 
-        table_pos = table_pos[-2:]
+            if "ViewEntry" in attr.get("href", "") and \
+                    attr.get("href") != last_entry:
+                cover = {"source": self.ROOT_URL}
+                last_entry = attr.get("href")
 
-        if len(table_pos) != 2:
-            return
+            if cover:
+                if attr.get("src") and "thumbnail" not in cover:
+                    cover["thumbnail"] = attr["src"]
 
-        tr_pos = []
-        for i in xrange(table_pos[0], table_pos[1]):
-            if self.data[i][0] == 'tr' and not self.data[i][1]:
-                tr_pos.append(i)
+                    uid = attr["src"].rsplit("/")[-1].split(".")[0]
+                    url = self.ROOT_URL + "/res/exe/GetElement.php?ID=" + uid
+                    cover["cover"] = url
 
-        for tr in xrange(0, len(tr_pos) - 1):
-            cover = {}
+                if data and "name" not in cover:
+                    cover["name"] = data
 
-            album = self.data[tr_pos[tr] : tr_pos[tr + 1]]
+                if "dimension" in old_data.lower() and data:
+                    cover["resolution"] = data
 
-            name = []
-            for i, entry in enumerate(album):
-                if entry[0] == 'b':
-                    for a in xrange(i, len(album)):
-                        name.append(album[a][2])
-                        if album[a + 1][0] == 'b':
-                            break
-                    break
+                if "filesize" in old_data.lower() and data:
+                    cover["size"] = data
 
-            cover['name'] = ''.join(name).strip()
+                if len(cover.keys()) >= 6:
+                    covers.append(cover)
+                    cover = None
 
-            sub_data = [i[2] for i in album if i[0] == '' and i[2].strip()]
+            old_data = data
 
-            cover['size'] = sub_data[-1].strip().replace(',', '.')
+        return covers
 
-            cover['resolution'] = sub_data[-2].strip()
-
-            thumb_tag = [i[1] for i in album if i[0] == 'img'][0]
-            cover['thumbnail'] = thumb_tag['src']
-
-            id = cover['thumbnail'].split("/")[-1].split(".")[0]
-            cover['cover'] = self.root_url + "/res/exe/GetElement.php?ID=" + id
-
-            cover['source'] = self.root_url
-
-            self.covers.append(cover)
-
-            if(len(self.covers) >= self.limit):
-                break
-
-    def __parse_page_num(self):
-        """Tries to find the number of found covers on an already parsed
-        search result page and how many results are on one page
-        (needed for result/page offset)"""
-
-        pos = 0
-
-        for i, entry in enumerate(self.data):
-            if entry[0] == 'div' and 'class' in entry[1] and \
-                entry[1]['class'] == 'BoxHeadline':
-                pos = i
-                break
-
-        text = self.data[pos][2]
-        nums = [int(i) for i in text.split() if i.isdigit()][-2:]
-
-        if len(nums) == 2:
-            self.page_step, self.cover_count = nums
-            return True
-        else:
-            return False
 
 class DiscogsParser(object):
     """A class for searching covers from discogs.com"""
@@ -489,6 +432,10 @@ class AmazonParser(object):
         parameters['SearchIndex'] = 'Music'
         parameters['Keywords'] = query
         parameters['ItemPage'] = page
+        # This specifies where the money goes and needed since 1.11.2011
+        # (What a good reason to break API..)
+        # ...so use the gnome.org one
+        parameters['AssociateTag'] = 'gnomestore-20'
 
         data, enc = get_url(url, get=parameters)
         dom = minidom.parseString(data)
@@ -573,12 +520,12 @@ class AmazonParser(object):
 class CoverArea(gtk.VBox):
     """The image display and saving part."""
 
-    def __init__(self, parent, dirname):
+    def __init__(self, parent, song):
         super(CoverArea, self).__init__()
-
+        self.song = song
         self.connect('destroy', self.__save_config)
 
-        self.dirname = dirname
+        self.dirname = song("~dirname")
         self.main_win = parent
 
         self.data_cache = []
@@ -590,6 +537,9 @@ class CoverArea(gtk.VBox):
         self.button.set_sensitive(False)
         self.button.connect('clicked', self.__save)
 
+        close_button = gtk.Button(stock=gtk.STOCK_CLOSE)
+        close_button.connect('clicked', lambda x: self.main_win.destroy())
+
         self.window_fit = gtk.CheckButton(_('Fit image to _window'))
         self.window_fit.connect('toggled', self.__scale_pixbuf)
 
@@ -617,6 +567,26 @@ class CoverArea(gtk.VBox):
         #create the filename combo box
         fn_list = ['cover.jpg', 'folder.jpg', '.folder.jpg']
 
+        # Issue 374 - add dynamic file names
+        artist = song("artist")
+        alartist = song("albumartist")
+        album = song("album")
+        labelid = song("labelid")
+        if album:
+            fn_list.append("<album>.jpg")
+            if alartist:
+                fn_list.append("<albumartist> - <album>.jpg")
+            else:
+                fn_list.append("<artist> - <album>.jpg")
+        else:
+            print_w("No album for \"%s\". Could be difficult finding art..." %
+                    song("~filename"))
+            title = song("title")
+            if title and artist:
+                fn_list.append("<artist> - <title>.jpg")
+        if (labelid):
+            fn_list.append("<labelid>.jpg")
+
         set_fn = cfg_get('fn', fn_list[0])
 
         for i, fn in enumerate(fn_list):
@@ -644,11 +614,18 @@ class CoverArea(gtk.VBox):
         self.scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
 
         bbox = gtk.HButtonBox()
+        bbox.set_spacing(6)
+        bbox.set_layout(gtk.BUTTONBOX_END)
         bbox.pack_start(self.button)
+        bbox.pack_start(close_button)
+
+        bb_align = gtk.Alignment(0, 1, 1, 0)
+        bb_align.set_property('right-padding', 6)
+        bb_align.add(bbox)
 
         main_hbox = gtk.HBox()
-        main_hbox.pack_start(table, False, padding=5)
-        main_hbox.pack_start(bbox)
+        main_hbox.pack_start(table, False, padding=6)
+        main_hbox.pack_start(bb_align)
 
         top_hbox = gtk.HBox()
         top_hbox.pack_start(self.open_check)
@@ -673,6 +650,9 @@ class CoverArea(gtk.VBox):
         """save the cover, spawn the program to edit it if selected"""
 
         filename = self.name_combo.get_active_text()
+        # Allow support for filename patterns
+        pattern = Pattern(filename)
+        filename = util.fsencode(pattern.format(self.song))
         file_path = os.path.join(self.dirname, filename)
 
         if os.path.exists(file_path) and not qltk.ConfirmAction(None,
@@ -861,7 +841,7 @@ class AlbumArtWindow(qltk.Window):
         self.set_icon_name(gtk.STOCK_FIND)
         self.set_default_size(800, 550)
 
-        image = CoverArea(self, songs[0]('~dirname'))
+        image = CoverArea(self, songs[0])
 
         self.liststore = gtk.ListStore(gtk.gdk.Pixbuf, object)
         self.treeview = treeview = AllTreeView(self.liststore)
@@ -1085,26 +1065,14 @@ class CoverSearch(object):
         """Creates searching threads which call the callback function after
         they are finished"""
 
-        global debug
-
         clean_query = self.__cleanup_query(query, replace)
 
-        #Catch exceptions and print them in the warning console.
-        #Some engines might break over time since the web interfaces
-        #could change.
-
-        if debug:
-            eng_instance = engine()
-            result = eng_instance.start(clean_query, self.overall_limit)
-        else:
-            try:
-                eng_instance = engine()
-                result = eng_instance.start(clean_query, self.overall_limit)
-            except Exception, msg:
-                text = _('[AlbumArt] %s failed: "%s", "%s"') \
-                    % (engine.__name__, query, msg)
-                print_w(text)
-                result = []
+        result = []
+        try:
+            result = engine().start(clean_query, self.overall_limit)
+        except Exception:
+            print_w("[AlbumArt] %s: %r" % (engine.__name__, query))
+            util.print_exc()
 
         self.finished += 1
         #progress is between 0..1
@@ -1199,13 +1167,13 @@ eng['config_id'] = 'amazon'
 engines.append(eng)
 
 #-------
-eng = {}
-eng['class'] = DiscogsParser
-eng['url'] = 'http://www.discogs.com/'
-eng['replace'] = ' '
-eng['config_id'] = 'discogs'
+#eng = {}
+#eng['class'] = DiscogsParser
+#eng['url'] = 'http://www.discogs.com/'
+#eng['replace'] = ' '
+#eng['config_id'] = 'discogs'
 
-engines.append(eng)
+#engines.append(eng)
 #------------------------------------------------------------------------------
 
 def change_config(checkb, id):
@@ -1223,23 +1191,24 @@ class DownloadAlbumArt(SongsMenuPlugin):
     def PluginPreferences(klass, window):
         global engines, change_config, config_eng_prefix
 
-        vbox = gtk.VBox(spacing=5)
-        vbox.set_border_width(5)
+        table = gtk.Table(len(engines), 2)
+        table.set_col_spacings(6)
+        table.set_row_spacings(6)
 
-        for eng in engines:
+        frame = qltk.Frame(_("Sources"), child=table)
+
+        for i, eng in enumerate(sorted(engines, key=lambda x: x["url"])):
             check = gtk.CheckButton(eng['config_id'].title())
-            vbox.pack_start(check)
-            check.connect('toggled', change_config, eng['config_id'])
-
+            table.attach(check, 0, 1, i, i + 1)
             checked = cfg_get(config_eng_prefix + eng['config_id'], True)
-
             check.set_active(checked)
+            check.connect('toggled', change_config, eng['config_id'])
 
             button = gtk.Button(eng['url'])
             button.connect('clicked', lambda s:util.website(s.get_label()))
-            vbox.pack_start(button)
+            table.attach(button, 1, 2, i, i + 1, xoptions=gtk.FILL | gtk.SHRINK)
 
-        return vbox
+        return frame
 
     PluginPreferences = classmethod(PluginPreferences)
 
