@@ -15,8 +15,15 @@ import time
 import gtk
 import gobject
 import pango
-from musicbrainz2 import webservice as ws
-from musicbrainz2.utils import extractUuid
+
+try:
+    from musicbrainz2 import webservice as ws
+    from musicbrainz2.utils import extractUuid
+except ImportError:
+    from quodlibet import plugins
+    if not hasattr(plugins, "PluginImportException"): raise
+    raise plugins.PluginImportException(
+        "Couldn't find python-musicbrainz2.")
 
 from quodlibet import config, util
 from quodlibet.qltk.ccb import ConfigCheckButton
@@ -48,6 +55,11 @@ def config_get(key, default=''):
     except config.error:
         return default
 
+def dialog_get_widget_for_stockid(dialog, stockid):
+    for child in dialog.get_action_area().get_children():
+        if child.get_label() == stockid:
+            return child
+
 class ResultTreeView(HintedTreeView, MultiDragTreeView):
     """The result treeview. The model only stores local tracks; info about
     remote results is pulled from self.remote_album."""
@@ -264,7 +276,8 @@ class SearchWindow(gtk.Dialog):
             if (config_get('albumartist', True)
                 and extractUuid(album.artist.id) != VARIOUS_ARTISTS_ARTISTID):
                 shared['albumartist'] = album.artist.name
-                if config_get('artist_sort', True):
+                if config_get('artist_sort', False) and \
+                        album.artist.sortName != album.artist.name:
                     shared['albumartistsort'] = album.artist.sortName
 
         if config_get('standard', True):
@@ -283,11 +296,13 @@ class SearchWindow(gtk.Dialog):
                 song['musicbrainz_trackid'] = extractUuid(track.id)
             if album.isSingleArtistRelease() or not track.artist:
                 song['artist'] = album.artist.name
-                if config_get('artist_sort', True):
+                if config_get('artist_sort', False) and \
+                        album.artist.sortName != album.artist.name:
                     song['artistsort'] = album.artist.sortName
             else:
                 song['artist'] = track.artist.name
-                if config_get('artist_sort', True):
+                if config_get('artist_sort', False) and \
+                        track.artist.sortName != track.artist.name:
                     song['artistsort'] = track.artist.sortName
                 if config_get('standard', True):
                     song['musicbrainz_artistid'] = extractUuid(track.artist.id)
@@ -356,7 +371,8 @@ class SearchWindow(gtk.Dialog):
         self.result_treeview.update_remote_album(release.tracks)
         self.current_release = release
         self.release_combo.update(release)
-        self.get_action_area().get_children()[1].set_sensitive(True)
+        save_button = dialog_get_widget_for_stockid(self, gtk.STOCK_SAVE)
+        save_button.set_sensitive(True)
 
     def __init__(self, album, cache):
         self.album = album
@@ -373,6 +389,9 @@ class SearchWindow(gtk.Dialog):
         self.set_default_size(650, 500)
         self.set_border_width(5)
 
+        save_button = dialog_get_widget_for_stockid(self, gtk.STOCK_SAVE)
+        save_button.set_sensitive(False)
+
         vb = gtk.VBox()
         vb.set_spacing(8)
 
